@@ -30,6 +30,7 @@
 #include "utils/nt_std_t.h"
 #include "utils/text.h"
 #include "utils/file.h"
+#include "utils/nt_pthread.h"
 #include "utils/base64.h"
 #include "utils/crypt.h"
 #include "utils/nt_conv_char.h"
@@ -42,6 +43,10 @@ static BOOL nt_cloud_edit_lines_file(nt_cloud_handle handle,
 static BOOL nt_cloud_upload_file_local(nt_cloud_handle handle,
 		const char *file_name, nt_link_tp lines, int depth);
 static nt_cloud_handle g_cloud_handle = NULL;
+static nt_pthread_result_t upload_file_pthread_func(void *param);
+static BOOL nt_cloud_edit_lines_file_async(nt_cloud_handle handle,
+		const char *file_name, nt_link_tp lines, const char *php_file);
+static nt_pthread_result_t edit_lines_file_pthread_func(void *param);
 
 nt_cloud_handle nt_cloud_get_handle(){
 	if(!g_cloud_handle)
@@ -215,6 +220,102 @@ BOOL nt_cloud_delete_lines_from_file(nt_cloud_handle handle,
 {
 	return nt_cloud_edit_lines_file(handle, file_name, lines, NT_CLOUD_DELETE_LINES_FILE_PHP, 0);
 }
+BOOL nt_cloud_insert_lines_into_file_async(nt_cloud_handle handle,
+		const char *file_name, nt_link_tp lines)
+{
+	return nt_cloud_edit_lines_file_async(
+			handle, file_name, lines, NT_CLOUD_INSERT_LINES_FILE_PHP);
+}
+BOOL nt_cloud_delete_lines_from_file_async(nt_cloud_handle handle,
+		const char *file_name, nt_link_tp lines)
+{
+	return nt_cloud_edit_lines_file_async(
+			handle, file_name, lines, NT_CLOUD_DELETE_LINES_FILE_PHP);
+}
+
+
+static BOOL nt_cloud_edit_lines_file_async(nt_cloud_handle handle,
+		const char *file_name, nt_link_tp lines, const char *php_file)
+{
+	nt_link_tp param;
+	nt_pthread_handle h_pthread;
+	char *cptr;
+	
+	cptr = nt_str_clone(file_name);
+	if(!cptr){
+		return FALSE;
+	}
+	
+	param = nt_link_add_data(NULL, handle);
+	if(!param){
+		free(cptr);
+		return FALSE;
+	}
+	if(NULL == nt_link_add_data(param, cptr)){
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	if(NULL == nt_link_add_data(param, php_file)){
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	if(NULL == nt_link_add_data(param, lines)){
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	nt_cloud_add_ref(handle);
+	
+	h_pthread = nt_pthread_alloc(
+			edit_lines_file_pthread_func, param, NULL);
+	if(!h_pthread){
+		nt_cloud_release_ref(handle);
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	
+	if(!nt_pthread_put_que(h_pthread)){
+		nt_pthread_release_ref(h_pthread);
+		nt_cloud_release_ref(handle);
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	nt_pthread_release_ref(h_pthread);
+	return TRUE;
+}
+
+static nt_pthread_result_t edit_lines_file_pthread_func(void *param)
+{
+	nt_pthread_result_t result;
+	nt_link_tp linkp, lines;
+	char *file_name, *php_file_name;
+	nt_cloud_handle h_cloud;
+	
+	
+	result.code = 0;
+	result.data = NULL;
+	
+	linkp = (nt_link_tp)param;
+	if(4 != nt_link_num(linkp)){
+		nt_all_link_free(linkp, NULL);
+		return result;
+	}
+	h_cloud = linkp->data;
+	file_name = linkp->next->data;
+	php_file_name = linkp->next->next->data;
+	lines = linkp->next->next->next->data;
+	(void)nt_cloud_edit_lines_file(h_cloud,
+			file_name, lines, php_file_name, 0);
+	nt_cloud_release_ref(h_cloud);
+	nt_all_link_free(lines, free);
+	nt_all_link_free(linkp, NULL);
+	free(file_name);
+	return result;
+}
 
 static BOOL nt_cloud_edit_lines_file(nt_cloud_handle handle,
 		const char *file_name, nt_link_tp lines, const char *php_file, int depth)
@@ -430,6 +531,84 @@ BOOL nt_cloud_upload_file(nt_cloud_handle handle,
 			file_name, lines, 0);
 }
 
+BOOL nt_cloud_upload_file_async(nt_cloud_handle handle,
+		const char *file_name, nt_link_tp lines)
+{
+	nt_link_tp param;
+	nt_pthread_handle h_pthread;
+	char *cptr;
+	
+	cptr = nt_str_clone(file_name);
+	if(!cptr){
+		return FALSE;
+	}
+	
+	param = nt_link_add_data(NULL, handle);
+	if(param){
+		free(cptr);
+		return FALSE;
+	}
+	if(NULL == nt_link_add_data(param, cptr)){
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	if(NULL == nt_link_add_data(param, lines)){
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	nt_cloud_add_ref(handle);
+	
+	h_pthread = nt_pthread_alloc(
+			upload_file_pthread_func, param, NULL);
+	if(!h_pthread){
+		nt_cloud_release_ref(handle);
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	
+	if(!nt_pthread_put_que(h_pthread)){
+		nt_pthread_release_ref(h_pthread);
+		nt_cloud_release_ref(handle);
+		nt_all_link_free(param, NULL);
+		free(cptr);
+		return FALSE;
+	}
+	nt_pthread_release_ref(h_pthread);
+	return TRUE;
+}
+
+static nt_pthread_result_t upload_file_pthread_func(void *param)
+{
+	nt_pthread_result_t result;
+	nt_link_tp linkp, lines;
+	char *file_name;
+	nt_cloud_handle h_cloud;
+	
+	
+	result.code = 0;
+	result.data = NULL;
+	
+	linkp = (nt_link_tp)param;
+	if(3 != nt_link_num(linkp)){
+		nt_all_link_free(linkp, NULL);
+		return result;
+	}
+	h_cloud = linkp->data;
+	file_name = linkp->next->data;
+	lines = linkp->next->next->data;
+	(void)nt_cloud_upload_file_local(h_cloud,
+			file_name, lines, 0);
+	nt_cloud_release_ref(h_cloud);
+	nt_all_link_free(lines, free);
+	nt_all_link_free(linkp, NULL);
+	free(file_name);
+	return result;
+}
+
+
 static BOOL nt_cloud_upload_file_local(nt_cloud_handle handle,
 		const char *file_name, nt_link_tp lines, int depth)
 {
