@@ -27,6 +27,10 @@ extern char USR_LOG_DB_PATH[];
 extern char USR_FAVORITE_BOARD_FILE_PATH[];
 extern char USR_FAVORITE_GRP_FILE_PATH[];
 extern char USR_FAVORITE_THREAD_FILE_PATH[];
+extern char USR_NG_WORD_FILE_PATH[];
+extern char USR_NG_NAME_FILE_PATH[];
+extern char USR_NG_ID_FILE_PATH[];
+
 extern char *RFC2898_SALT;
 extern int  RFC2898_ITERATION;
 extern char *AES256_PASS;
@@ -80,9 +84,11 @@ extern char *MARU_PW;
 #define NT_KEY_COMMAND1 ':' 
 #define NT_KEY_COMMAND2 '/' 
 #define NT_KEY_COMMAND3 '?'
+#define NT_KEY_CMD_BASE (65536)
 #define NT_KEY_CMD_BOARD_UPDATE (65536 + 1)
 #define NT_KEY_CMD_AUTO_SCROLL (65536 + 2)
 #define NT_KEY_CMD_FAVORITE_UPDATE (65536 + 3)
+#define NT_KEY_CMD_NGWORD_UPDATE (65536 + 4)
 
 #define NT_COMMAND1_WRITE_MSG_1 "write"
 #define NT_COMMAND1_WRITE_MSG_2 "w"
@@ -115,8 +121,15 @@ extern char *MARU_PW;
 #define NT_COMMAND1_UPLOAD_BOARD2 "b"
 #define NT_COMMAND1_UPLOAD_THREAD1 "thread"
 #define NT_COMMAND1_UPLOAD_THREAD2 "t"
+#define NT_COMMAND1_UPLOAD_NG_FILE1 "ng"
+#define NT_COMMAND1_UPLOAD_NG_FILE2 "n"
 #define NT_COMMAND1_LIMIT_1 "limit"
 #define NT_COMMAND1_LIMIT_2 "l"
+#define NT_COMMAND1_NG_WORD_1 "ngword"
+#define NT_COMMAND1_NG_WORD_2 "ngwd"
+#define NT_COMMAND1_NG_NAME_1 "ngname"
+#define NT_COMMAND1_NG_NAME_2 "ngnm"
+#define NT_COMMAND1_NG_ID_1 "ngid"
 
 extern int set_option(int argc, char* argv[]);
 
