@@ -26,6 +26,7 @@ typedef struct tag_nt_usr_db_thread_data_t *nt_usr_db_thread_data_tp;
 typedef struct tag_nt_usr_db_thread_data_t{
      int read_count;
 	 wchar_t *dat_name;
+	 wchar_t *board_name;
 }nt_usr_db_thread_data_t;
 
 extern void nt_usr_db_thread_data_free(void *ptr);
@@ -49,6 +50,8 @@ extern BOOL nt_usr_db_delete_thread_log(nt_usr_db_handle db_handle,
 	nt_link_tp *thread_data_list);
 extern char* nt_usr_db_board_last_query(nt_usr_db_handle db_handle,
 	const wchar_t *board_name);
+extern nt_link_tp nt_usr_db_query_usr_table(nt_usr_db_handle db_handle,
+	int offset, int limit);
 
 
 #endif /* __USR_DB_T_H_ */
