@@ -47,24 +47,19 @@
 #include "ui/disp.h"
 #include "ui/disp_win.h"
 #include "ui/disp_string.h"
-#ifdef NT_CLOUD
 #include "cloud/nt_cloud.h"
-#endif
+#include "usr/ng_word_t.h"
+
 #define S_SIZE 	(1024)
 
 extern int server_main();
 
 static int draw_title(WINDOW *wp, const wchar_t *title, attr_t attr);
-#ifdef NT_CLOUD
 static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 			nt_cloud_handle h_cloud,
 			nt_favorite_handle h_favorite, 
-			nt_favorite_grp_handle h_favorite_grp);
-#else
-static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
-			nt_favorite_handle h_favorite, 
-			nt_favorite_grp_handle h_favorite_grp);
-#endif
+			nt_favorite_grp_handle h_favorite_grp,
+			nt_ng_word_handle h_ng_word);
 static void print_error(WINDOW *wp, const wchar_t *msg);
 
 static void _2ch_selected_item_free(void *ptr)
@@ -83,10 +78,9 @@ int main(int argc, char *argv[])
 	nt_favorite_grp_handle h_favorite_grp;
 	nt_link_tp text_linkp, text2_linkp;
 	const char *err_msg;
-#ifdef NT_CLOUD
+	nt_ng_word_handle h_ng_word;
 	nt_cloud_handle h_cloud;
 	h_cloud = NULL;
-#endif
 	err_msg = NULL;
 
 	setlocale(LC_ALL, "ja_JP.UTF-8");
@@ -129,7 +123,6 @@ int main(int argc, char *argv[])
 		goto ERROR_TRAP;
 	}
 	
-#ifdef NT_CLOUD
 	if(nt_crypt_lib_init(RFC2898_SALT, RFC2898_ITERATION, 
 			AES256_PASS, &err_msg)){
 		if(NCE_AUTH_URL && NCE_ID && NCE_PASS){
@@ -141,7 +134,6 @@ int main(int argc, char *argv[])
 			sleep(1);
 		}
 	}
-#endif
 
 	if(!nt_2ch_model_init()){
 		fputs("Failed to read board menu data.\n", stderr);
@@ -167,11 +159,9 @@ int main(int argc, char *argv[])
 	}
 	
 	text_linkp = NULL;
-#ifdef NT_CLOUD
 	if(h_cloud){
 		text_linkp = nt_cloud_download_file(h_cloud, "fb.txt");
 	}
-#endif
 	if(!text_linkp)
 		text_linkp = nt_read_text_file(USR_FAVORITE_BOARD_FILE_PATH);
 	if(text_linkp){
@@ -179,13 +169,11 @@ int main(int argc, char *argv[])
 		nt_all_link_free(text_linkp, free);
 	}
 	text_linkp = NULL;
-#ifdef NT_CLOUD
 	if(h_cloud){
 		text_linkp = nt_cloud_download_file(h_cloud, "ft.txt");
 		nt_cloud_query_favorite_attributes_async(h_cloud, usr_db_handle);
 		//nt_cloud_query_attributes(h_cloud, usr_db_handle, 0);
 	}
-#endif
 	if(!text_linkp)
 		text_linkp = nt_read_text_file(USR_FAVORITE_THREAD_FILE_PATH);
 	if(text_linkp){
@@ -193,19 +181,28 @@ int main(int argc, char *argv[])
 		nt_all_link_free(text_linkp, free);
 	}
 	
+	h_ng_word = nt_ng_word_load(h_cloud,
+			USR_NG_WORD_FILE_PATH,
+			USR_NG_NAME_FILE_PATH,
+			USR_NG_ID_FILE_PATH);
+	
 	cbreak();
 	noecho();
 
-#ifdef NT_CLOUD
-	if(DoLoop(scrp, usr_db_handle, h_cloud, h_favorite, h_favorite_grp))
-#else
-	if(DoLoop(scrp, usr_db_handle, h_favorite, h_favorite_grp))
-#endif
+	if(DoLoop(scrp, usr_db_handle, h_cloud, h_favorite, h_favorite_grp, h_ng_word))
 		result = 0;
 
 	echo();
 	nocbreak();
 	
+	if(h_ng_word){
+		nt_ng_word_save(h_ng_word,
+			USR_NG_WORD_FILE_PATH,
+			USR_NG_NAME_FILE_PATH,
+			USR_NG_ID_FILE_PATH);
+		nt_ng_word_release_ref(h_ng_word);
+	}
+	
 	text_linkp = nt_favorite_retrieve_boards(h_favorite);
 	if(text_linkp){
 		nt_write_text_file(USR_FAVORITE_BOARD_FILE_PATH, text_linkp);
@@ -244,11 +241,9 @@ ERROR_TRAP:
 
 	nt_timer_lib_finish();
 	
-#ifdef NT_CLOUD
 	nt_crypt_lib_finish();
 	if(h_cloud)
 		nt_cloud_release_ref(h_cloud);
-#endif
 	nt_env_free();
 	return (result);
 }
@@ -267,16 +262,11 @@ static int auto_update_timer_func(int id)
 	return TIMER_ID_AUTO_UPDATE_NONE;
 }
 
-#ifdef NT_CLOUD
 static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 		nt_cloud_handle h_cloud,
 		nt_favorite_handle h_favorite, 
-		nt_favorite_grp_handle h_favorite_grp)
-#else
-static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
-		nt_favorite_handle h_favorite, 
-		nt_favorite_grp_handle h_favorite_grp)
-#endif
+		nt_favorite_grp_handle h_favorite_grp,
+		nt_ng_word_handle h_ng_word)
 {
 	int ch, state, num;
 	int disp_state, nresult;
@@ -369,14 +359,12 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 					h_favorite_board = nt_favorite_board_alloc(
 						h_favorite, board_name);
 					if(h_favorite_board){
-#ifdef NT_CLOUD
 						linkp = nt_link_add_data(NULL, (void*)board_name);
 						if(linkp){
 							if(h_cloud)
 								nt_cloud_insert_lines_into_file(h_cloud, "fb.txt", linkp);
 							free(linkp);
 						}
-#endif
 						nt_favorite_board_release_ref(h_favorite_board);
 						//favorite_dump(h_favorite);
 					}
@@ -423,14 +411,12 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 						nt_2ch_selected_item_get_board_name(h_sel_items),
 						nt_2ch_selected_item_get_thread_title(h_sel_items));
 					if(h_favorite_thread){
-#ifdef NT_CLOUD
 						linkp = NULL;
 						if(nt_favorite_retrieve_thread(h_favorite_thread, &linkp)){
 							if(h_cloud)
 								nt_cloud_insert_lines_into_file(h_cloud, "ft.txt", linkp);
 							nt_all_link_free(linkp, free);
 						}
-#endif
 						nt_favorite_thread_release_ref(h_favorite_thread);
 						//favorite_dump(h_favorite);
 						status_msg = NT_INFO_ADD_FAVORITE_SUCCEEDED;
@@ -507,7 +493,7 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 						LINES - num, COLS, num, 0))
 				goto END_WHILE;
 			rwinp->key = ch;
-			state = disp_reslist(rwinp, state, h_sel_items, db_handle);
+			state = disp_reslist(rwinp, state, h_sel_items, db_handle, h_ng_word, h_cloud);
 			auto_scrolling = FALSE;
 			if(DISP_CMD(state)){
 				if(DISP_CMD(state) == DISP_CMD_AUTO_SCROLL){
@@ -516,6 +502,7 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 							h_timer_auto_scroll, NT_AUTO_SCROLL_INTERVAL);
 					}
 					auto_scrolling = TRUE;
+					state = DISP_STATE_RESLIST;
 				}else if(DISP_CMD(state) == DISP_CMD_REFRESH){
 					state = DISP_STATE_RESLIST;
 					ch = NT_KEY_NONE;
@@ -531,8 +518,50 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 					status_msg = NT_INFO_REFRESH_THREAD_SUCCESS;
 					wclear(scrp);
 					continue;
+				}else if(DISP_CMD(state) == DISP_CMD_REENTER){
+					DISP_CLR_CMD(state);
+					ch = NT_KEY_NONE;
+					continue;
+				}else if(DISP_CMD(state) == DISP_CMD_EDIT_NGWORD){
+					ch = NT_KEY_NONE;
+					if(!h_ng_word)
+						break;
+					linkp = nt_ng_word_get_words(h_ng_word);
+					linkp = disp_editor3(linkp);
+					if(linkp && nt_ng_word_set_words(h_ng_word, linkp, h_cloud)){
+						ch = NT_KEY_CMD_NGWORD_UPDATE;
+					}
+					DISP_CLR_CMD(state);
+					keypad(stdscr, true);
+					wclear(scrp);
+					continue;
+				}else if(DISP_CMD(state) == DISP_CMD_EDIT_NGNAME){
+					ch = NT_KEY_NONE;
+					if(!h_ng_word)
+						break;
+					linkp = nt_ng_word_get_names(h_ng_word);
+					linkp = disp_editor3(linkp);
+					if(linkp && nt_ng_word_set_names(h_ng_word, linkp, h_cloud)){
+						ch = NT_KEY_CMD_NGWORD_UPDATE;
+					}
+					DISP_CLR_CMD(state);
+					keypad(stdscr, true);
+					wclear(scrp);
+					continue;
+				}else if(DISP_CMD(state) == DISP_CMD_EDIT_NGID){
+					ch = NT_KEY_NONE;
+					if(!h_ng_word)
+						break;
+					linkp = nt_ng_word_get_ids(h_ng_word);
+					linkp = disp_editor3(linkp);
+					if(linkp && nt_ng_word_set_ids(h_ng_word, linkp, h_cloud)){
+						ch = NT_KEY_CMD_NGWORD_UPDATE;
+					}
+					DISP_CLR_CMD(state);
+					keypad(stdscr, true);
+					wclear(scrp);
+					continue;
 				}
-				DISP_CLR_CMD(state);
 			}
 			if(!auto_scrolling){
 				nt_timer_set_interval(h_timer_auto_scroll, -1);
@@ -723,7 +752,6 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 				if(DISP_CMD(state) == DISP_CMD_DEL_FAVORITE_BOARD){
 					if(nt_favorite_board_remove(handle)){
 						h_favorite_board = (nt_favorite_board_handle)handle;
-#ifdef NT_CLOUD
 						if(h_cloud){
 							linkp = nt_link_add_data(NULL, 
 									(void*)nt_favorite_board_get_name(h_favorite_board));
@@ -732,7 +760,6 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 								free(linkp);
 							}
 						}
-#endif
 						nt_favorite_board_release_ref(h_favorite_board);
 						status_msg = NT_INFO_DEL_FAVORITE_SUCCEEDED;
 					}else{
@@ -741,7 +768,6 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 				}else if(DISP_CMD(state) == DISP_CMD_DEL_FAVORITE_THREAD){
 					if(nt_favorite_thread_remove(handle)){
 						h_favorite_thread = (nt_favorite_thread_handle)handle;
-#ifdef NT_CLOUD
 						if(h_cloud){
 							linkp = NULL;
 							if(nt_favorite_retrieve_thread(h_favorite_thread, &linkp)){
@@ -749,7 +775,6 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 								nt_all_link_free(linkp, free);
 							}
 						}
-#endif
 						nt_favorite_thread_release_ref(h_favorite_thread);
 						status_msg = NT_INFO_DEL_FAVORITE_SUCCEEDED;
 					}else{
@@ -786,7 +811,6 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 					status_msg = NT_INFO_REFRESH_FAVORITE_SUCCESS;
 					wclear(scrp);
 					continue;
-#ifdef NT_CLOUD
 				}else if(h_cloud){
 					if(DISP_CMD(state) == DISP_CMD_UPLOAD_ALL ||
 							DISP_CMD(state) == DISP_CMD_UPLOAD_BOARDS){
@@ -813,8 +837,13 @@ static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 							}
 						}
 					}
+					if(DISP_CMD(state) == DISP_CMD_UPLOAD_ALL ||
+							DISP_CMD(state) == DISP_CMD_UPLOAD_NG_FILES){
+						if(nt_ng_word_upload_cloud(h_cloud, h_ng_word)){
+							status_msg = NT_INFO_UPLOAD_FAVORITE_SUCCEEDED;
+						}
+					}
 					disp_state = state = DISP_STATE_FAVORITE;
-#endif
 				}else{
 					disp_state = state = DISP_STATE_FAVORITE;
 				}
