@@ -29,12 +29,81 @@
 #include "_2ch/_2ch.h"
 #include "utils/nt_std_t.h"
 #include "utils/text.h"
+#include "utils/file.h"
 #include "ui/disp.h"
 
 
 static BOOL read_editor(FILE *fp, nt_write_data_handle h_write_data,
 		char *buf_reuse, size_t buf_len);
 
+BOOL disp_editor2(const char *file_name)
+{
+	char buf[1024*2];
+	int ret;
+	
+	sprintf(buf, "%s %s", EDITOR_CMD, file_name);
+	ret = system(buf);
+	if (WIFSIGNALED(ret) &&
+         	(WTERMSIG(ret) == SIGINT || 
+			 WTERMSIG(ret) == SIGQUIT))
+		return FALSE;
+	return TRUE;
+}
+
+BOOL nt_link_cmp(nt_link_tp l, nt_link_tp r, nt_compare_fn cmp_func)
+{
+	nt_link_tp lp, rp;
+	lp = l;
+	rp = r;
+	do{
+		if(0 != cmp_func(lp->data, rp->data))
+			return FALSE;
+		lp = lp->next;
+		rp = rp->next;
+	}while(l != lp || r != rp);
+	return (l == lp && r == rp);
+}
+
+nt_link_tp disp_editor3(nt_link_tp text_list)
+{
+	FILE *tmp_fp;
+	pid_t pid;
+	char fname[512];
+	char buf[128];
+	int ret;
+	nt_link_tp out_linkp;
+	pid = getpid();
+	
+	sprintf(fname, "%s/txt%d.tmp", LOG_PATH, pid);
+	if(!text_list){
+		tmp_fp = fopen(fname, "w");
+		if(!tmp_fp)
+			return FALSE;
+		fclose(tmp_fp);
+	}else{
+		if(!nt_write_text_file(fname, text_list))
+			return NULL;
+	}
+	
+	sprintf(buf, "%s %s", EDITOR_CMD, fname);
+	ret = system(buf);
+	if (WIFSIGNALED(ret) &&
+         	(WTERMSIG(ret) == SIGINT || 
+			 WTERMSIG(ret) == SIGQUIT))
+		return NULL;
+	
+	out_linkp =  nt_read_text_file(fname);
+	if(!out_linkp)
+		return NULL;
+	
+	if(nt_link_cmp(out_linkp, text_list, nt_link_wcscmp_fnc)){
+		nt_all_link_free(out_linkp, free);
+		return NULL;
+	}
+	return out_linkp;
+}
+
+
 BOOL disp_editor(nt_write_data_handle h_write_data)
 {
 	FILE *tmp_fp;
