@@ -37,6 +37,7 @@
 #define NT_CMD_UPLOAD_BOARDS 2
 #define NT_CMD_UPLOAD_THREADS 3
 #define NT_CMD_DISP_HISTORY 4
+#define NT_CMD_UPLOAD_NG_FILES 5
 
 typedef struct tag_favorite_ctx_t *favorite_ctx_tp;
 typedef struct tag_favorite_ctx_t {
@@ -121,6 +122,8 @@ int disp_favorite(nt_window_tp wp,
 			return DISP_CMD_UPLOAD_BOARDS;
 		case NT_CMD_UPLOAD_THREADS:
 			return DISP_CMD_UPLOAD_THREADS;
+		case NT_CMD_UPLOAD_NG_FILES:
+			return DISP_CMD_UPLOAD_NG_FILES;
 		case NT_CMD_DISP_HISTORY:
 			return DISP_STATE_HISTORY;
 		}
@@ -491,6 +494,9 @@ static int parse_cmd1(const char *param)
 	}else if(0 == strncmp(start, NT_COMMAND1_UPLOAD_THREAD1,len) ||
 		0 == strncmp(start, NT_COMMAND1_UPLOAD_THREAD2,len)){
 		return NT_CMD_UPLOAD_THREADS;
+	}else if(0 == strncmp(start, NT_COMMAND1_UPLOAD_NG_FILE1,len) ||
+		0 == strncmp(start, NT_COMMAND1_UPLOAD_NG_FILE2,len)){
+		return NT_CMD_UPLOAD_NG_FILES;
 	}
 	
 	return NT_CMD_ERR;
