@@ -43,6 +43,12 @@
 #define NT_CMD_FAVORITE 6
 #define NT_CMD_HISTORY 7
 #define NT_CMD_AUTO_SCROLL 8
+#define NT_CMD_ADD_NGWORD 9
+#define NT_CMD_ADD_NGNAME 10
+#define NT_CMD_ADD_NGID 11
+#define NT_CMD_EDT_NGWORD 12
+#define NT_CMD_EDT_NGNAME 13
+#define NT_CMD_EDT_NGID 14
 
 #define AUTO_SCROLL_NONE 0
 #define AUTO_SCROLL_UP  -1
@@ -91,18 +97,21 @@ static BOOL reslist_clone(nt_thread_handle h_thread, ctx_reslist_tp ctxp);
 static void int_ptr_free(void *ptr);
 static int parse_cmd1(const char *param, const char **end);
 static BOOL search_line_asc(regex_t *regexp, nt_link_tp reslistp, 
-			int *sel_res_no, int *sel_res_line, int column);
+			int *sel_res_no, int *sel_res_line, int column, 
+			nt_ng_word_handle h_ngword);
 static BOOL search_line_desc(regex_t *regexp, ctx_reslist_tp ctxp,
 			nt_link_tp reslistp, 
-			int *sel_res_no, int *sel_res_line, int column);
+			int *sel_res_no, int *sel_res_line, int column,
+			nt_ng_word_handle h_ngword);
 static int parse_res_msg(nt_link_tp disp_res_list, 
-			res_data_tp res_datap, size_t colmns);
+			res_data_tp res_datap, size_t colmns, nt_ng_word_handle h_ngword);
 static void res_msg_free(void *ptr);
 static nt_link_tp get_cited_num_list(nt_link_tp res_listp, int seq_no);
 static BOOL reslist_copy(ctx_reslist_tp ctxp, ctx_reslist_tp copy_ctxp, 
 			nt_link_tp num_listp);
 static ctx_reslist_tp init_sub_context(ctx_reslist_tp ctxp, nt_link_tp num_listp);
 static void free_reslist_sub_ctx(void *ptr);
+static void free_disp_lines_ctx(void *ptr);
 static nt_link_tp parse_tree_list(ctx_reslist_tp ctxp, const char *param);
 static nt_link_tp parse_id_list(ctx_reslist_tp ctxp, const char *param);
 static void search_up_tree(int seq_no, nt_link_tp disp_list, nt_link_tp *num_linkp);
@@ -112,7 +121,7 @@ static BOOL set_res_header(ctx_reslist_tp ctxp, res_data_tp res_datap,
 				wchar_t *buf, size_t buf_len);
 
 int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_select,
-	nt_usr_db_handle usr_db_handle)
+	nt_usr_db_handle usr_db_handle, nt_ng_word_handle h_ng_word, nt_cloud_handle h_cloud)
 {
 	ctx_reslist_tp ctxp, child_ctxp;
 	res_data_tp res_datap;
@@ -154,7 +163,8 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 		}
 	}
 	
-	if(prev_state != DISP_STATE_RESLIST)
+	if(prev_state != DISP_STATE_RESLIST &&
+		prev_state < NT_KEY_CMD_BASE)
 		ctxp->prev_state = prev_state;
 
 	ch = wp->key;
@@ -168,7 +178,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 		while(clistp != ctxp->res_disp_list){
 			res_datap = (res_data_tp)clistp->data;
 			if(!res_datap->msg_line_linkp){
-				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 			}
 
 			if(res_datap->msg_header_line_num <= 0){
@@ -344,14 +354,14 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 		if(search_asc){
 			if(search_line_asc(&(ctxp->regex),ctxp->res_disp_list,
 					&ctxp->sel_res_no, &ctxp->sel_res_line, 
-					wp->cols - 5)){
+					wp->cols - 5, h_ng_word)){
 				adjust = TRUE;
 			}
 		}else{
 			if(search_line_desc(&(ctxp->regex),ctxp,
 					ctxp->res_disp_list,
 					&ctxp->sel_res_no, &ctxp->sel_res_line, 
-					wp->cols - 5)){
+					wp->cols - 5, h_ng_word)){
 				adjust = TRUE;
 			}
 		}
@@ -366,7 +376,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 					res_datap = (res_data_tp)clistp->data;
 					//resp = (nt_res_tp)clistp->data;
 					if(!res_datap->msg_line_linkp){
-						parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+						parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 					}
 					if(res_datap->msg_header_line_num <= 0){
 						if(!set_res_header((ctx_reslist_tp)wp->data, 
@@ -447,6 +457,27 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 			}
 			ctxp->cur_res_offset = 0;
 			break;
+		case NT_CMD_ADD_NGWORD:
+			nt_ng_word_add_ng_word(h_cloud, h_ng_word, endp);
+			free_disp_lines_ctx(wp->data);
+			return DISP_CMD_REENTER | DISP_STATE_RESLIST;
+		case NT_CMD_ADD_NGNAME:
+			nt_ng_word_add_ng_name(h_cloud, h_ng_word, endp);
+			free_disp_lines_ctx(wp->data);
+			return DISP_CMD_REENTER | DISP_STATE_RESLIST;
+		case NT_CMD_ADD_NGID:
+			nt_ng_word_add_ng_id(h_cloud, h_ng_word, endp);
+			free_disp_lines_ctx(wp->data);
+			return DISP_CMD_REENTER | DISP_STATE_RESLIST;
+		case NT_CMD_EDT_NGWORD:
+			free_disp_lines_ctx(wp->data);
+			return DISP_CMD_EDIT_NGWORD | DISP_STATE_RESLIST;
+		case NT_CMD_EDT_NGNAME:
+			free_disp_lines_ctx(wp->data);
+			return DISP_CMD_EDIT_NGNAME | DISP_STATE_RESLIST;
+		case NT_CMD_EDT_NGID:
+			free_disp_lines_ctx(wp->data);
+			return DISP_CMD_EDIT_NGID | DISP_STATE_RESLIST;
 		default:
 			if(cmd == NT_CMD_TREE){
 				linkp = parse_tree_list((ctx_reslist_tp)wp->data, endp);
@@ -516,7 +547,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 			break;
 		res_datap = (res_data_tp)clistp->data;
 		if(!res_datap->msg_line_linkp){
-			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 		}
 		if(res_datap->msg_header_line_num <= 0){
 			if(!set_res_header((ctx_reslist_tp)wp->data, 
@@ -557,7 +588,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 			break;
 		res_datap = (res_data_tp)clistp->data;
 		if(!res_datap->msg_line_linkp){
-			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 		}
 		if(res_datap->msg_header_line_num <= 0){
 			if(!set_res_header((ctx_reslist_tp)wp->data, 
@@ -608,7 +639,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 		for( ; i > 0; i--){
 			res_datap = (res_data_tp)clistp->data;
 			if(!res_datap->msg_line_linkp){
-				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 			}
 			if(res_datap->msg_header_line_num <= 0){
 				if(!set_res_header((ctx_reslist_tp)wp->data, 
@@ -665,7 +696,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 		for( ; i < ctxp->res_num; i++){
 			res_datap = (res_data_tp)clistp->data;
 			if(!res_datap->msg_line_linkp){
-				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 			}
 			if(res_datap->msg_header_line_num <= 0){
 				if(!set_res_header((ctx_reslist_tp)wp->data, 
@@ -768,7 +799,7 @@ int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_
 		if(rows == wp->lines)
 			break;
 		if(!res_datap->msg_line_linkp){
-			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
+			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5, h_ng_word);
 		}
 		if(0 == res_datap->msg_line_num)
 			continue;
@@ -935,6 +966,35 @@ static int parse_cmd1(const char *param, const char **end)
 		0 == strncmp(NT_COMMAND1_AUTOSCROLL_2,param,
 			strlen(NT_COMMAND1_AUTOSCROLL_2))){
 		return NT_CMD_AUTO_SCROLL;
+	}else if(0 == strncmp(NT_COMMAND1_NG_WORD_1,param,len) ||
+			0 == strncmp(NT_COMMAND1_NG_WORD_2,param, len)){
+		if(!nt_strtok(*end, ' ', &start, end))
+			return NT_CMD_EDT_NGWORD;
+		
+		len = *end - start;
+		if(len <= 0)
+			return NT_CMD_EDT_NGWORD;
+		*end = start;
+		return NT_CMD_ADD_NGWORD;
+	}else if(0 == strncmp(NT_COMMAND1_NG_NAME_1,param,len) ||
+			0 == strncmp(NT_COMMAND1_NG_NAME_2,param, len)){
+		if(!nt_strtok(*end, ' ', &start, end))
+			return NT_CMD_EDT_NGNAME;
+		
+		len = *end - start;
+		if(len <= 0)
+			return NT_CMD_EDT_NGNAME;
+		*end = start;
+		return NT_CMD_ADD_NGNAME;
+	}else if(0 == strncmp(NT_COMMAND1_NG_ID_1,param,len)){
+		if(!nt_strtok(*end, ' ', &start, end))
+			return NT_CMD_EDT_NGID;
+		
+		len = *end - start;
+		if(len <= 0)
+			return NT_CMD_EDT_NGID;
+		*end = start;
+		return NT_CMD_ADD_NGID;
 	}
 	return NT_CMD_NONE;
 }
@@ -1035,6 +1095,20 @@ static ctx_reslist_tp init_sub_context(ctx_reslist_tp ctxp, nt_link_tp num_listp
 	return child_ctxp;
 }
 
+static const wchar_t *get_id(const wchar_t *source)
+{
+	const wchar_t *cptr;
+	if(!source)
+		return NULL;
+	cptr = wcsstr(source, L"ID:");
+	if(!cptr)
+		return NULL;
+	cptr += 3;
+	if(cptr[0] == L'\0' || cptr[0] == L'?')
+		return NULL;
+	return cptr;
+}
+
 static BOOL reslist_clone(nt_thread_handle h_thread, ctx_reslist_tp ctxp)
 {
 	nt_enum_handle h_enum_res;
@@ -1131,6 +1205,36 @@ void free_reslist_ctx(void *ptr)
 	}
 	free(ptr);
 }
+
+static void free_disp_lines_ctx(void *ptr)
+{
+	ctx_reslist_tp ctxp;
+	nt_stack_tp stackp;
+	nt_link_tp linkp;
+	res_data_tp res_datap;
+	if(!ptr)
+		return;
+	ctxp = (ctx_reslist_tp)ptr;
+	if(ctxp->res_disp_list){
+		linkp = ctxp->res_disp_list;
+		do{
+			res_datap = (res_data_tp)linkp->data;
+			if(res_datap->msg_line_linkp){
+				free(res_datap->msg_line_linkp->data);
+				nt_all_link_free(res_datap->msg_line_linkp, NULL);
+				res_datap->msg_line_linkp = NULL;
+			}
+			res_datap->msg_line_num = 0;
+			linkp = linkp->next;
+		}while(linkp != ctxp->res_disp_list);
+	}
+	stackp = ctxp->child_ctx_stackp;
+	if(stackp){
+		nt_stack_free(stackp, free_reslist_sub_ctx);
+		ctxp->child_ctx_stackp = NULL;
+	}
+}
+
 static void free_reslist_sub_ctx(void *ptr)
 {
 	ctx_reslist_tp ctxp;
@@ -1223,11 +1327,14 @@ static nt_link_tp get_cited_num_list(nt_link_tp res_listp, int seq_no)
 }
 
 static int parse_res_msg(nt_link_tp disp_res_list, 
-				res_data_tp res_datap, size_t colmns)
+				res_data_tp res_datap, size_t colmns,
+				nt_ng_word_handle h_ngword)
 {
-	int i, len, start, lines;
+	int i, len, start, lines, nwrite;
 	wchar_t ch;
 	wchar_t *buf, *cptr, *srcp, *wrk_buf;
+	const wchar_t *name, *id, *misc;
+	wchar_t match[256];
 	int offset;
 	nt_link_tp linkp;
 	int consume_colmns;
@@ -1240,8 +1347,47 @@ static int parse_res_msg(nt_link_tp disp_res_list,
 
 	//res_msg_free(res_datap);
 	res_datap->msg_line_num = 0;
-
-	len = wcslen(res_datap->res_msg);
+	
+	name = nt_res_get_name(res_datap->h_res);
+	misc = nt_res_get_misc(res_datap->h_res);
+	if(misc)
+		id = get_id(misc);
+	else
+		id = NULL;
+	cptr = NULL;
+	if(name && NG_ITEM_NAME == 
+			ng_word_match(h_ngword, NG_ITEM_NAME, name, 
+			match, sizeof(match)/sizeof(wchar_t))){
+		cptr = L"NGNAME:";
+	}else if(id && NG_ITEM_ID == 
+			ng_word_match(h_ngword, NG_ITEM_ID, id, 
+			match, sizeof(match)/sizeof(wchar_t))){
+		cptr = L"NGID:";
+	}else if(NG_ITEM_MSG == ng_word_match(h_ngword, NG_ITEM_MSG, 
+			res_datap->res_msg, 
+			match, sizeof(match)/sizeof(wchar_t))){
+		cptr = L"NGWORD:";
+	}
+	if(cptr){
+		len = wcslen(cptr) + wcslen(match) + 1;
+		buf = malloc(len*sizeof(wchar_t));
+		if(!buf)
+			return 0;
+		swprintf(buf, len, L"%ls%ls", cptr, match);
+		nwrite = nt_get_wc_count_within_colmns(buf, colmns);
+		if(nwrite <= 0){
+			free(buf);
+			return 0;
+		}
+		buf[nwrite] = L'\0';
+		res_datap->msg_line_num = 1;
+		res_datap->msg_line_linkp = nt_link_add_data(NULL, buf);
+		return 1;
+	}
+	
+	srcp = res_datap->res_msg;
+	
+	len = wcslen(srcp);
 	if(len == 0)
 		return 0;
 	buf_size = len + delta;
@@ -1253,7 +1399,6 @@ static int parse_res_msg(nt_link_tp disp_res_list,
 		return 0;
 	buf_idx = 0;
 
-	srcp = res_datap->res_msg;
 
 	lines = 0;
 	start = 0;
@@ -1340,7 +1485,7 @@ END_FOR:
 }
 
 static BOOL search_line_asc(regex_t *regexp, nt_link_tp reslistp, 
-			int *sel_res_no, int *sel_res_line, int column)
+			int *sel_res_no, int *sel_res_line, int column, nt_ng_word_handle h_ngword)
 {
 	nt_link_tp clistp, listp;
 	//nt_res_tp resp;
@@ -1370,7 +1515,7 @@ static BOOL search_line_asc(regex_t *regexp, nt_link_tp reslistp,
 			res_datap = (res_data_tp)clistp->data;
 			//resp = res_datap->resp;
 			if(!res_datap->msg_line_linkp)
-				parse_res_msg(reslistp, res_datap, column);
+				parse_res_msg(reslistp, res_datap, column, h_ngword);
 			listp = res_datap->msg_line_linkp;
 			line = 0;
 			do{
@@ -1402,7 +1547,7 @@ static BOOL search_line_asc(regex_t *regexp, nt_link_tp reslistp,
 
 		res_datap = (res_data_tp)clistp->data;
 		if(!res_datap->msg_line_linkp)
-			parse_res_msg(reslistp, res_datap, column);
+			parse_res_msg(reslistp, res_datap, column, h_ngword);
 		listp = res_datap->msg_line_linkp;
 		line = 0;
 		do{
@@ -1429,7 +1574,8 @@ static BOOL search_line_asc(regex_t *regexp, nt_link_tp reslistp,
 
 static BOOL search_line_desc(regex_t *regexp, ctx_reslist_tp ctxp,
 			nt_link_tp reslistp, 
-			int *sel_res_no, int *sel_res_line, int column)
+			int *sel_res_no, int *sel_res_line, int column, 
+			nt_ng_word_handle h_ngword)
 {
 	nt_link_tp clistp, listp;
 	//nt_res_tp resp;
@@ -1461,7 +1607,7 @@ static BOOL search_line_desc(regex_t *regexp, ctx_reslist_tp ctxp,
 			res_datap = (res_data_tp)clistp->data;
 			//resp = res_datap->resp;
 			if(!res_datap->msg_line_linkp)
-				parse_res_msg(reslistp, res_datap, column);
+				parse_res_msg(reslistp, res_datap, column, h_ngword);
 			listp = res_datap->msg_line_linkp->prev;
 			line = res_datap->msg_line_num - 1;
 			do{
@@ -1494,7 +1640,7 @@ static BOOL search_line_desc(regex_t *regexp, ctx_reslist_tp ctxp,
 		res_datap = (res_data_tp)clistp->data;
 		//resp = (nt_res_tp)clistp->data;
 		if(!res_datap->msg_line_linkp)
-			parse_res_msg(reslistp, res_datap, column);
+			parse_res_msg(reslistp, res_datap, column, h_ngword);
 		listp = res_datap->msg_line_linkp->prev;
 		line = res_datap->msg_line_num - 1;
 		do{
