@@ -78,6 +78,7 @@ static BOOL get_thread_data_by_dat_name(const wchar_t *dat_name, nt_link_tp disp
 #define NT_CMD_DEL_THREAD_LOG 4
 #define NT_CMD_SEARCH_THREAD 5
 #define NT_CMD_FAVORITE 6
+#define NT_CMD_HISTORY 7
 
 static int parse_cmd1(const char *param);
 static thread_disp_data_tp get_thread_by_seq_no(nt_link_tp linkp, int seq_no);
@@ -183,8 +184,10 @@ int disp_threadlist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle
 
 	switch(ch){
 	case NT_KEY_REFRESH: 
-		result_state = DISP_STATE_REFRESH;
+		result_state = DISP_CMD_REFRESH;
 		goto ERROR_TRAP;
+	case NT_KEY_ERASE:
+		return DISP_STATE_BOARDMENU; 
 	case NT_KEY_CLOSE:
 	case KEY_LEFT:
 		result_state = ctxp->prev_state;
@@ -242,17 +245,17 @@ int disp_threadlist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle
 			case NT_CMD_SORT_NUMBER:
 				THREAD_SORT_TYPE = NT_THREAD_SORT_BY_NUMBER;
 				sort_thread_title(THREAD_SORT_TYPE, ctxp);
-				result_state = DISP_STATE_REFRESH;
+				result_state = DISP_CMD_REFRESH;
 				goto ERROR_TRAP;
 			case NT_CMD_SORT_READ:
 				THREAD_SORT_TYPE = NT_THREAD_SORT_BY_READ;
 				sort_thread_title(THREAD_SORT_TYPE, ctxp);
-				result_state = DISP_STATE_REFRESH;
+				result_state = DISP_CMD_REFRESH;
 				goto ERROR_TRAP;
 			case NT_CMD_SORT_UNREAD:
 				THREAD_SORT_TYPE = NT_THREAD_SORT_BY_UNREAD;
 				sort_thread_title(THREAD_SORT_TYPE, ctxp);
-				result_state = DISP_STATE_REFRESH;
+				result_state = DISP_CMD_REFRESH;
 				goto ERROR_TRAP;
 			case NT_CMD_DEL_THREAD_LOG: 
 				if(!nt_strtok(wp->cmd_param,' ', &start, &end))
@@ -274,7 +277,7 @@ int disp_threadlist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle
 						}
 						nt_cloud_release_ref(h_cloud);
 					}
-					result_state = DISP_STATE_REFRESH;
+					result_state = DISP_CMD_REFRESH;
 					goto ERROR_TRAP;
 				}
 				linkp = nt_parse_number_list(cptr, NULL);
@@ -306,7 +309,7 @@ int disp_threadlist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle
 				if(h_cloud)
 					nt_cloud_release_ref(h_cloud);
 				nt_all_link_free(linkp, NULL);
-				result_state = DISP_STATE_REFRESH;
+				result_state = DISP_CMD_REFRESH;
 				goto ERROR_TRAP;
 			case NT_CMD_SEARCH_THREAD:
 				result_state = DISP_STATE_SEARCH_THREAD;
@@ -314,6 +317,9 @@ int disp_threadlist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle
 			case NT_CMD_FAVORITE:
 				result_state = DISP_STATE_FAVORITE;
 				goto ERROR_TRAP;
+			case NT_CMD_HISTORY:
+				result_state = DISP_STATE_HISTORY;
+				goto ERROR_TRAP;
 			default:
 				break;
 			}
@@ -854,6 +860,11 @@ static int parse_cmd1(const char *param)
 		0 == strncmp(NT_COMMAND1_FAVORITE_2,param,
 			strlen(NT_COMMAND1_FAVORITE_2))){
 		return NT_CMD_FAVORITE;
+	}else if(0 == strncmp(NT_COMMAND1_HISTORY_1,param,
+			strlen(NT_COMMAND1_HISTORY_1)) ||
+		0 == strncmp(NT_COMMAND1_HISTORY_2,param,
+			strlen(NT_COMMAND1_HISTORY_2))){
+		return NT_CMD_HISTORY;
 	}else if(0 == strncmp(NT_COMMAND1_SORT_1, param,
 			strlen(NT_COMMAND1_SORT_1))){
 		offset = strlen(NT_COMMAND1_SORT_1);
