@@ -1345,7 +1345,7 @@ nt_link_tp nt_favorite_get_update_board_list(
 	nt_board_handle h_board;
 	nt_mutex_handle h_mutex;
 	nt_link_tp res_linkp, tmp_linkp, board_name_linkp;
-	const wchar_t *board_name, *cptr;
+	const wchar_t *board_name;
 	
 	h_mutex = nt_favorite_get_mutex(h_favorite);
 	if(!h_mutex){
@@ -1372,10 +1372,10 @@ nt_link_tp nt_favorite_get_update_board_list(
 			if(!board_name)
 				continue;
 			if(board_name_linkp){
-				cptr = (const wchar_t*)nt_link_find(
-						board_name_linkp, (void*)board_name, nt_link_wcscmp_fnc);
-				if(cptr)
+				if(nt_link_find(board_name_linkp, 
+						(void*)board_name, nt_link_wcscmp_fnc)){
 					continue;
+				}
 			}
 			//fwprintf(stderr, L"%ls\n", board_name);
 			h_board = nt_get_board_by_name(h_model, board_name, &h_category);
