@@ -54,7 +54,6 @@ static BOOL nt_pthread_exec(nt_pthread_handle h_pthread);
 static nt_pthread_result_t nt_pthread_call_result_func(nt_pthread_handle h_pthread);
 static BOOL nt_pthread_has_result(nt_pthread_handle h_pthread);
 
-
 BOOL nt_pthread_lib_init(int thread_pool_size, int queue_size,
 		const char **error_msg)
 {
@@ -112,6 +111,9 @@ void nt_pthread_lib_finish()
 	free(g_sem.pthreads);
 	pthread_mutex_destroy(&(g_sem.mutex));
 	sem_destroy(&g_sem.sem);
+	
+	nt_queue_free(g_sem.h_que, NULL);
+	nt_queue_free(g_sem.h_que_result, NULL);
 }
 
 BOOL nt_pthread_put_que(nt_pthread_handle h_pthread)
@@ -131,6 +133,8 @@ BOOL nt_pthread_put_que(nt_pthread_handle h_pthread)
 	if(!nt_queue_push(g_sem.h_que, h_pthread))
 		goto ERROR_TRAP;
 	
+	nt_pthread_add_ref(h_pthread);
+	
 	err = sem_post(&g_sem.sem);
 	if(err != 0)
 		goto ERROR_TRAP;
