@@ -31,7 +31,7 @@ wchar_t* nt_w_str_resize(wchar_t *old_src, size_t old_size, size_t new_size)
 	
 	assert(old_size < new_size && new_size > 0);
 	
-	wrk_buf = malloc(sizeof(wchar_t) * (new_size));
+	wrk_buf = malloc(sizeof(wchar_t) * (new_size+1));
 	if(!wrk_buf){
 		return NULL;
 	}
@@ -136,7 +136,7 @@ wchar_t *nt_w_trim(const wchar_t *source)
 	}/* end for */
 	assert(i <= j);
 	len = j - i + 1;
-	cptr = malloc(sizeof(wchar_t)*(len + 1));
+	cptr = malloc(sizeof(wchar_t)*(len + 2));
 	if(cptr != NULL){
 		memcpy(cptr, source+i, len*sizeof(wchar_t));
 		cptr[len] = '\0';
@@ -152,6 +152,7 @@ wchar_t *nt_w_trim_quotes(const wchar_t *source)
 	wchar_t *cptr;
 	BOOL dquote = FALSE;
 	BOOL squote = FALSE;
+	size_t buf_len;
 
 	for(i=0; i<len; i++){
 		switch(source[i]){
@@ -218,7 +219,11 @@ wchar_t *nt_w_trim_quotes(const wchar_t *source)
 	}/* end for */
 	len = j - i + 1;
 	if(i < j){
-		cptr = malloc(sizeof(wchar_t)*(len + 1));
+		buf_len = len + 1;
+		buf_len *= sizeof(wchar_t);
+		if(buf_len % 8)
+			buf_len += 8 - (buf_len%8);
+		cptr = malloc(buf_len);
 		if(cptr != NULL){
 			memcpy(cptr, source+i, len*sizeof(wchar_t));
 			cptr[len] = L'\0';
@@ -243,7 +248,7 @@ char *nt_trim2(const char *source, char *buf, size_t buf_len)
 	len = strlen(source);
 
 	if(!buf){
-		cptr = calloc(len+1, sizeof(char));
+		cptr = calloc(len+2, sizeof(char));
 		if(!cptr)
 			return NULL;
 	}else{
@@ -295,7 +300,14 @@ char *nt_trim2(const char *source, char *buf, size_t buf_len)
 wchar_t* nt_w_str_clone(const wchar_t* src)
 {
 	wchar_t *cptr;
-	cptr = malloc(sizeof(wchar_t) * (wcslen(src)+1));
+	size_t len;
+	
+	len = wcslen(src) + 1;
+	len *= sizeof(wchar_t);
+	if(len % 8){
+		len += 8 - (len % 8);
+	}
+	cptr = malloc(len);
 	if(!cptr)
 		return NULL;
 	wcscpy(cptr, src);
@@ -305,7 +317,13 @@ wchar_t* nt_w_str_clone(const wchar_t* src)
 char* nt_str_clone(const char* src)
 {
 	char *cptr;
-	cptr = malloc(strlen(src)+1);
+	size_t len;
+	
+	len = strlen(src) + 1;
+	if(len % 8){
+		len += 8 - (len % 8);
+	}
+	cptr = malloc(len);
 	if(!cptr)
 		return NULL;
 	strcpy(cptr, src);
@@ -316,12 +334,16 @@ char* nt_substr(const char* src, int start, int end)
 {
 	char *cptr;
 	int len;
+	size_t buf_len;
 
 	assert(start >= 0);
 	assert(start <= end);
 
 	len = end - start;
-	cptr = malloc(len + 1);
+	buf_len = len + 1;
+	if(buf_len % 8)
+		buf_len += 8 - (buf_len % 8);
+	cptr = malloc(buf_len);
 	if(!cptr)
 		return NULL;
 	
@@ -339,12 +361,17 @@ wchar_t* nt_w_substr(const wchar_t* src, int start, int end)
 {
 	wchar_t *cptr;
 	int len;
+	size_t buf_len;
 
 	assert(start >= 0);
 	assert(start <= end);
 
 	len = end - start;
-	cptr = malloc((len + 1)*sizeof(wchar_t));
+	buf_len = len + 1;
+	buf_len *= sizeof(wchar_t);
+	if(buf_len % 8)
+		buf_len += 8 - (buf_len % 8);
+	cptr = malloc(buf_len);
 	if(!cptr)
 		return NULL;
 	
